#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* warning this code is also in getclasses.cxx under pcmaker */
/* this roputine creates the init file */
static void CreateInitFile(const char *libName, 
  int numConcrete, char **concrete, 
  FILE *fout) 
{
  int i;

#if defined(_WIN32) && !defined(__CYGWIN__)
  const char* prefix = "";
#else
  const char* prefix = "lib";
#endif

#if defined(_WIN32)
  const char* dllexp = "__declspec(dllexport) ";
#else
  const char* dllexp = "";
#endif
  
  fprintf(fout,"// Generated by vtkWrapRubyInit in VTK/Wrapping\n");
  fprintf(fout,"#include \"vtkRuby.h\"\n\n");
  fprintf(fout,"#include \"vtkSystemIncludes.h\"\n");
  fprintf(fout,"#include <string.h>\n");
  fprintf(fout,"// Handle compiler warning messages, etc.\n"
          "#if defined( _MSC_VER ) && !defined(VTK_DISPLAY_WIN32_WARNINGS)\n"
          "#pragma warning ( disable : 4706 )\n"
          "#endif // Windows Warnings \n\n");
  
for (i = 0; i < numConcrete; i++)
    {
    fprintf(fout,"extern  \"C\" {%svoid Init_%s(VALUE); }\n", dllexp, concrete[i]);
    }
  
#ifdef _WIN32
  fprintf(fout,"extern  \"C\" {%svoid Init_%s();}\n\n", dllexp, libName);
  fprintf(fout,"void Init_%s()\n{\n", libName);
#else
  fprintf(fout,"extern  \"C\" {%svoid Init_lib%s();}\n\n", dllexp, libName);
  fprintf(fout,"void Init_lib%s()\n{\n", libName);
#endif
  
  /* module init function */
  fprintf(fout,"  VALUE mVtk = rb_define_module(\"Vtk\");\n\n");

  for (i = 0; i < numConcrete; i++)
    {
    fprintf(fout,"  Init_%s(mVtk);\n", concrete[i]);
    }
  fprintf(fout,"}\n\n");
}


int main(int argc,char *argv[])
{
  FILE *file;
  FILE *fout;
  int numConcrete = 0;
  char libName[250];
  char tmpVal[250];
  char *concrete[4000];

  if (argc < 3)
    {
    fprintf(stderr,"Usage: %s input_file output_file\n",argv[0]);
    return 1;
    }

  file = fopen(argv[1],"r");
  if (!file) 
    {
    fprintf(stderr,"Input file %s could not be opened\n",argv[1]);
    return 1;
    }

  fout = fopen(argv[2],"w");
  if (!fout)
    {
    return 1;
    }

  /* read the info from the file */
  fscanf(file,"%s",libName);

  /* read in the classes */
  while (fscanf(file,"%s",tmpVal) != EOF)
    {
    concrete[numConcrete] = strdup(tmpVal);
    numConcrete++;
    }
  /* close the file */
  fclose(file);

  CreateInitFile(libName, numConcrete, concrete, fout);
  fclose(fout);

  return 0;
}

